#version 450
#extension GL_ARB_separate_shader_objects : enable

layout(binding = 0) uniform sampler2D inTexSampler0;

#ifdef SAMPLERS_0RGB1ALPHA
layout(binding = 1) uniform sampler2D inTexSampler1;
#endif

layout(location = 0) in vec4 inColor;
layout(location = 1) in vec2 inTexCoord;
#ifdef EFFECT_AMBIENT_WITH_LIGHT
layout(location = 2) in vec4 inScreenCoord;
#endif
layout(location = 0) out vec4 outColor;

#ifdef EFFECT_AMBIENT_WITH_LIGHT	
layout(push_constant) uniform PushConstants {
	layout(offset = 64)  uint  u_lightScrPos;
	layout(offset = 68)  float u_lightRadius;
	layout(offset = 72)  float u_lightBorder;
	layout(offset = 76)  float u_lineEqRedKR;
	layout(offset = 80)  float u_lineEqRedKG;
	layout(offset = 84)  float u_lineEqRedKB;
	layout(offset = 88)  float u_lineEqRedB;
	layout(offset = 92)  float u_lineEqGreenKR;
	layout(offset = 96)  float u_lineEqGreenKG;
	layout(offset = 100) float u_lineEqGreenKB;
	layout(offset = 104) float u_lineEqGreenB;
	layout(offset = 108) float u_lineEqBlueKR;
	layout(offset = 112) float u_lineEqBlueKG;
	layout(offset = 116) float u_lineEqBlueKB;
	layout(offset = 120) float u_lineEqBlueB;
	layout(offset = 124) uint  u_screenSize;
} pushConstants;
#endif

#ifdef EFFECT_ALPHA_GRADIENT
layout(push_constant) uniform PushConstants {
	layout(offset = 64)  float u_horizontal;
	layout(offset = 68)  float u_leftNearScrPos;
	layout(offset = 72)  float u_leftFarScrPos;
	layout(offset = 76)  float u_rightNearScrPos;
	layout(offset = 80)  float u_rightFarScrPos;
	layout(offset = 84)  float u_leftNormKoef;
	layout(offset = 88)  float u_rightNormKoef;
	layout(offset = 92)  float u_koefA;
	layout(offset = 96)  float u_koefB;
	layout(offset = 100) float u_koefC;
} pushConstants;
#endif

#ifdef EFFECT_YUVY_TO_RGBA 
layout(push_constant) uniform PushConstants {
	layout(offset = 64)  float u_isAlpha;
} pushConstants;
#endif

void main() {
#ifdef SAMPLERS_DEFAULT
   outColor = texture(inTexSampler0, inTexCoord);
#endif
#ifdef SAMPLERS_0RGB1ALPHA
   vec4 text0Col      = texture(inTexSampler0,inTexCoord );
   outColor           = texture(inTexSampler1,inTexCoord );
   outColor.a         = text0Col.r;
#endif

#ifdef EFFECT_YUVY_TO_RGBA 
  vec4 rgbaColor;
  rgbaColor.r = outColor.r + 1.13983 * (outColor.b - 0.5);
  rgbaColor.g = outColor.r - 0.39465 * (outColor.g - 0.5) - 0.58060 * (outColor.b - 0.5);
  rgbaColor.b = outColor.r + 2.03211 * (outColor.g - 0.5);

  if(pushConstants.u_isAlpha>0.0) {
	rgbaColor.a = (outColor.a - 0.0625)/0.8588;
  }
  else {
	rgbaColor.a = 1.0;
  }
  outColor = rgbaColor;

#endif

#ifdef EFFECT_COLOR_MULT
   outColor = outColor * inColor;   
#else
   outColor.a = outColor.a * inColor.a;   
#endif
   
#ifdef EFFECT_AMBIENT_WITH_LIGHT
   vec2 screenCoord = inScreenCoord.xy;
   uint lightScrPosX = pushConstants.u_lightScrPos >> 16;
   uint lightScrPosY = pushConstants.u_lightScrPos & 0xffff;
   uint screenWidth  = pushConstants.u_screenSize  >> 16;
   uint screenHeight = pushConstants.u_screenSize  & 0xffff;
   screenCoord.x = (screenCoord.x * 0.5 + 0.5) * screenWidth;
   screenCoord.y = (screenCoord.y * 0.5 + 0.5) * screenHeight;
         
   float dist = sqrt(pow(abs(lightScrPosX-screenCoord.x),2.0) + pow(abs(lightScrPosY-screenCoord.y),2.0));

   float resultR = pushConstants.u_lineEqRedKR   * outColor.r   +   pushConstants.u_lineEqRedKG   * outColor.g   +   pushConstants.u_lineEqRedKB   * outColor.b   +   pushConstants.u_lineEqRedB;
   float resultG = pushConstants.u_lineEqGreenKR * outColor.r   +   pushConstants.u_lineEqGreenKG * outColor.g   +   pushConstants.u_lineEqGreenKB * outColor.b   +   pushConstants.u_lineEqGreenB;
   float resultB = pushConstants.u_lineEqBlueKR  * outColor.r   +   pushConstants.u_lineEqBlueKG  * outColor.g   +   pushConstants.u_lineEqBlueKB  * outColor.b   +   pushConstants.u_lineEqBlueB;
   if(dist>pushConstants.u_lightRadius) {
	outColor.r = resultR;
	outColor.g = resultG;
	outColor.b = resultB;
   }
   else {
	if (dist>pushConstants.u_lightRadius - pushConstants.u_lightBorder)
	{
		float borderK = (dist - pushConstants.u_lightRadius + pushConstants.u_lightBorder)/pushConstants.u_lightBorder;

        outColor.r = (outColor.r*(1.0-borderK)) + (resultR*borderK);
        outColor.g = (outColor.g*(1.0-borderK)) + (resultG*borderK);
        outColor.b = (outColor.b*(1.0-borderK)) + (resultB*borderK);
	}
  }
#endif

#ifdef EFFECT_ALPHA_GRADIENT
  vec2 screenCoord = gl_FragCoord.xy;
	
  float curPos 	= pushConstants.u_horizontal==1.0?screenCoord.x:screenCoord.y;
	
  float leftNormDist 	= 0.0;	
  float rightNormDist 	= 0.0;	

  if ((pushConstants.u_leftNearScrPos!=pushConstants.u_leftFarScrPos) && curPos < pushConstants.u_leftNearScrPos && curPos >= pushConstants.u_leftFarScrPos) {
	leftNormDist = (curPos - pushConstants.u_leftNearScrPos)*pushConstants.u_leftNormKoef;
  }

  if ((pushConstants.u_rightNearScrPos!=pushConstants.u_rightFarScrPos) && curPos > pushConstants.u_rightNearScrPos && curPos <= pushConstants.u_rightFarScrPos) {
	rightNormDist = (curPos - pushConstants.u_rightNearScrPos)*pushConstants.u_rightNormKoef;
  }

  if (leftNormDist > 0.0) {
	outColor.a = outColor.a * ((leftNormDist*leftNormDist*pushConstants.u_koefA) + (pushConstants.u_koefB*leftNormDist) + pushConstants.u_koefC);
  } 

  if (rightNormDist > 0.0) {
	outColor.a = outColor.a * ((rightNormDist*rightNormDist*pushConstants.u_koefA) + (pushConstants.u_koefB*rightNormDist) + pushConstants.u_koefC);
  }   
#endif
}
